@@ -2,7 +2,7 @@
  * kernel/lvm-snap.c
  *
  * Copyright (C) 2000 Andrea Arcangeli <andrea@suse.de> SuSE
- *               2000 - 2001 Heinz Mauelshagen, Sistina Software
+ *               2000 - 2002 Heinz Mauelshagen, Sistina Software
  *
  * LVM snapshot driver is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
  *                 lvm_snapshot_fill_COW_table has a return value too.
  *    15/10/2001 - fix snapshot alignment problem [CM]
  *               - fix snapshot full oops (always check lv_block_exception) [CM]
+ *    26/06/2002 - support for new list_move macro [patch@luckynet.dynu.com]
  *
  */
 
@@ -65,6 +66,9 @@ static char *lvm_snap_version __attribute__ ((unused)) = "LVM "LVM_RELEASE_NAME"
 #ifndef LockPage
 #define LockPage(map) set_bit(PG_locked, &(map)->flags)
 #endif
+/* We need this for 2.2, ll_rw_blk validates against blksize rather than sectsize */
+#undef  lvm_sectsize
+#define lvm_sectsize(dev) lvm_get_blksize(dev)
 #endif
 
 extern const char *const lvm_name;
@@ -138,8 +142,11 @@ lvm_find_exception_table(kdev_t org_dev, unsigned long org_start, lv_t * lv)
 			if (i)
 			{
 				/* fun, isn't it? :) */
+#ifdef	list_move
+				list_move(next, hash_table);
+#else
 				list_del(next);
-				list_add(next, hash_table);
+#endif				list_add(next, hash_table);
 			}
 			ret = exception;
 			break;
@@ -365,7 +372,7 @@ int lvm_snapshot_COW(kdev_t org_phys_dev,
 	int idx = lv_snap->lv_remap_ptr, chunk_size = lv_snap->lv_chunk_size;
 	struct kiobuf * iobuf = lv_snap->lv_iobuf;
 #if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 4, 4)
-	static unsigned long blocklist[KIO_MAX_SECTORS];
+	unsigned long blocklist[KIO_MAX_SECTORS];
 	unsigned long *blocks = blocklist;
 #else
 	unsigned long *blocks = iobuf->blocks;
