@@ -5,7 +5,8 @@
  *
  * March,July,August 1999
  * January,April 2000
- * February 2001
+ * February,November 2001
+ * January 2002
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -35,6 +36,10 @@
  *    07/02/2001 - added COMPAQ_CISS & SMART2 major support
  *    17/08/2001 - added dynamic major support (Stefan Bader)
  *    10/09/2001 - fixed support for *non* partitioned MDs (Luca Bera)
+ *    27/11/2001 - added support for DAC960 in 2.2/2.4 (Adrian Phillips)
+ *    27/11/2001 - fixed lvm_get_device_type() to avoid name/type mismatch
+ *                 as that caused by the DAC960 fix (Andreas Dilger)
+ *    16/01/2002 - ATARAID support
  *
  */
 
@@ -55,8 +60,10 @@ int lvm_check_partitioned_dev(dev_t st_rdev)
     else {
 	switch (lvm_get_device_type(st_rdev)) {
 	case LVM_DEVICE_TYPE_INVALID:
-	    break;
+	case LVM_DEVICE_TYPE_NBD:
+	case LVM_DEVICE_TYPE_LOOP:
 	case LVM_DEVICE_TYPE_MD:
+	case LVM_DEVICE_TYPE_ATARAID:
 	    break;
 	default:
 	    ret = TRUE;
@@ -130,62 +137,62 @@ int lvm_check_extended_partition(dev_t st_rdev)
     return ret;
 }
 
-
+/*
+ * Given the device number, find out the type of a block device by looking
+ * in /proc/devices and matching the major numbers.
+ */
 const lvm_device_type_t lvm_get_device_type(dev_t device)
 {
     char line[80];
-    char *device_names[] = {
-	"ide",			/* IDE disk              */
-	"sd",			/* SCSI disk              */
-	"md",			/* Multiple Disk driver (SoftRAID)   */
-	"loop",			/* Loop device              */
-	"dasd",			/* DASD disk (IBM S/390, zSeries)      */
-	"dac960",		/* DAC960              */
-	"nbd",			/* Network Block Device              */
-	"ida",			/* Compaq SMART2              */
-	"cciss",		/* Compaq CCISS array              */
-	"ubd",			/* User-mode virtual block device */
-	NULL
+    struct devname_type {
+	char		 *name;
+	lvm_device_type_t type;
+    } device_names[] = {
+	{ "ide", LVM_DEVICE_TYPE_IDE },	  /* IDE disk			     */
+	{ "sd", LVM_DEVICE_TYPE_SD },	  /* SCSI disk			     */
+	{ "md", LVM_DEVICE_TYPE_MD },	  /* Multiple Disk driver (SoftRAID) */
+	{ "loop", LVM_DEVICE_TYPE_LOOP }, /* Loop device		     */
+	{ "dasd", LVM_DEVICE_TYPE_DASD }, /* DASD disk (IBM S/390, zSeries)  */
+	{ "rd", LVM_DEVICE_TYPE_DAC960 }, /* DAC960 (2.2 kernel name)	     */
+	{ "dac960", LVM_DEVICE_TYPE_DAC960 },/* DAC960 (2.4 kernel name)     */
+	{ "nbd", LVM_DEVICE_TYPE_NBD },	  /* Network Block Device	     */
+	{ "ida", LVM_DEVICE_TYPE_COMPAQ_SMART2 },/* Compaq SMART2	     */
+	{ "cciss", LVM_DEVICE_TYPE_COMPAQ_CISS },/* Compaq CISS array	     */
+	{ "ubd", LVM_DEVICE_TYPE_UBD },	  /* User-mode virtual block device  */
+	{ "ataraid", LVM_DEVICE_TYPE_ATARAID }, /* ATA Raid                  */
+	{ NULL, LVM_DEVICE_TYPE_INVALID } /* Device not recognized	     */
     };
     FILE *procdevices = NULL;
-    int ret = LVM_DEVICE_TYPE_INVALID;
-    int i, j = 0;
-    int seek_major = MAJOR(device);
-    int line_major = 0;
-    int blocksection = 0;
+    lvm_device_type_t ret = LVM_DEVICE_TYPE_INVALID;
 
     debug_enter("lvm_get_device_type called\n");
 
-    /* FIXME Ought to cache this first time */
     if ((procdevices = fopen("/proc/devices", "r")) != NULL) {
-	while (fgets(line, 80, procdevices) != NULL) {
-	    i = 0;
-	    while (line[i] == ' ' && line[i] != '\0')
-		i++;
+	int seek_major = MAJOR(device);
+	int blocksection = 0;
 
-	    /* If its not a number it may be the name of the section */
-	    line_major = atoi(((char *) (line + i)));
-	    if (line_major == 0) {
-		blocksection = (line[i] == 'B') ? 1 : 0;
+	while (fgets(line, 80, procdevices) != NULL) {
+	    char devname[NAME_LEN];
+	    unsigned line_major;
+
+	    if (memcmp(line, "Block", 5) == 0) {
+		blocksection = 1;
 		continue;
 	    }
 	    /* We only want block devices ... */
 	    if (!blocksection)
 		continue;
 
+	    if (sscanf(line, " %u %s", &line_major, devname) != 2)
+		    continue;
+
 	    if (line_major == seek_major) {
-		while (line[i] != ' ' && line[i] != '\0')
-		    i++;
-		while (line[i] == ' ' && line[i] != '\0')
-		    i++;
-		for (j = 0; device_names[j] != NULL; j++) {
-		    if (strlen(device_names[j]) <= strlen(line + i)) {
-			if (strncmp
-			    (device_names[j], line + i,
-			     strlen(device_names[j])) == 0) {
-			    ret = j;
+		int j;
+		for (j = 0; device_names[j].name != NULL; j++) {
+		    if (strncmp(device_names[j].name, devname,
+				strlen(device_names[j].name)) == 0) {
+			    ret = device_names[j].type;
 			    break;
-			}
 		    }
 		}
 		break;
@@ -193,6 +200,6 @@ const lvm_device_type_t lvm_get_device_type(dev_t device)
 	}
 	fclose(procdevices);
     }
-    debug_leave("lvm_get_device_type leaving with %i\n", ret);
+    debug_leave("lvm_get_device_type leaving with %d\n", ret);
     return (ret);
 }
