@@ -209,11 +209,14 @@ int vg_cfgbackup ( char* vg_name, char* directory,
       }
    }
 
-   sz = sprintf ( vg_backup_path, "%s/%s", directory, vg_name);
+   memset ( vg_backup_path, 0, sizeof ( vg_backup_path));
+   sz = snprintf ( vg_backup_path, sizeof ( vg_backup_path) - 1,
+                                  "%s/%s", directory, vg_name);
    if ( strcmp ( directory, VG_BACKUP_DIR) == 0)
-      sz += sprintf ( &vg_backup_path[sz], ".conf");
-   vg_backup_path[sz] = 0;
-   sprintf ( vg_backup_path_tmp, "%s.tmp", vg_backup_path);
+      sz += snprintf ( &vg_backup_path[sz], sizeof ( vg_backup_path) - sz - 1,
+                                            ".conf");
+   snprintf ( vg_backup_path_tmp, sizeof ( vg_backup_path_tmp) - 1,
+                                  "%s.tmp", vg_backup_path);
 
    if ( opt_v > 0) printf ( "%s -- checking existence of \"%s\"\n",
                             cmd, directory);
@@ -365,8 +368,10 @@ int vg_cfgbackup ( char* vg_name, char* directory,
 
       /* VGDA backup history */
       for ( i = vg_max_backups; i >= 1; i--) {
-         sprintf ( vg_backup_path_old2, "%s/%s.conf.%d.old%c",
-                                        directory, vg_name, i, 0);
+         memset ( vg_backup_path_old2, 0, sizeof ( vg_backup_path_old2));
+         snprintf ( vg_backup_path_old2, sizeof ( vg_backup_path_old2) - 1,
+                                         "%s/%s.conf.%d.old%c",
+                                         directory, vg_name, i, 0);
          if ( opt_v > 0) printf ( "%s -- checking for \"%s\"\n",
                                   cmd, vg_backup_path_old2);
          if ( i == vg_max_backups &&
@@ -379,10 +384,15 @@ int vg_cfgbackup ( char* vg_name, char* directory,
             }
          }
 
-         if ( i == 1) sprintf ( vg_backup_path_old1, "%s/%s.conf%c",
-                                        directory, vg_name, 0);
-         else sprintf ( vg_backup_path_old1, "%s/%s.conf.%d.old%c",
-                                             directory, vg_name, i - 1, 0);
+         memset ( vg_backup_path_old1, 0, sizeof ( vg_backup_path_old1));
+         if ( i == 1) snprintf ( vg_backup_path_old1,
+                                 sizeof ( vg_backup_path_old1) - 1,
+                                 "%s/%s.conf%c",
+                                 directory, vg_name, 0);
+         else snprintf ( vg_backup_path_old1,
+                         sizeof ( vg_backup_path_old1) - 1,
+                         "%s/%s.conf.%d.old%c",
+                         directory, vg_name, i - 1, 0);
          if ( opt_v > 0) printf ( "%s -- checking for \"%s\"\n",
                                   cmd, vg_backup_path_old1);
          if ( stat ( vg_backup_path_old1, &stat_b1) == -1) continue;
